//////////////////////////////////////////////////////////////////////////////////////////////
// Clase para el trabajar con matrices dispersas de punto flotante                          //
//                                                                                          //
// Anlisis y Diseo y Programacin:                                                        //
//                                                                                          //
// Nombre:   Antonio Carrillo Ledesma                                                       //
// E-mail:   acl@www.mmc.igeofcu.unam.mx                                                    //
// Pgina:   http://www.mmc.igeofcu.unam.mx/acl                                             //
//                                                                                          //
//                                                                                          //
// Este programa es software libre. Puede redistribuirlo y/o modificarlo                    //
// bajo los trminos de la Licencia Pblica General de GNU segn es                         //
// publicada por la Free Software Foundation, bien de la versin 2 de                       //
// dicha Licencia o bien (segn su eleccin) de cualquier versin                           //
// posterior.                                                                               //
//                                                                                          //
// Este programa se distribuye con la esperanza de que sea til, pero SIN                   //
// NINGUNA GARANTA, incluso sin la garanta MERCANTIL implcita o sin                      //
// garantizar la CONVENIENCIA PARA UN PROPSITO PARTICULAR. Vase la                        //
// Licencia Pblica General de GNU para ms detalles.                                       //
//                                                                                          //
// Debera haber recibido una copia de la Licencia Pblica General junto                    //
// con este programa. Si no ha sido as, escriba a la Free Software                         //
// Foundation, Inc., en 675 Mass Ave, Cambridge, MA 02139, EEUU.                            //
//                                                                                          //
//                                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////////



#ifndef __MatrizDispersa__
#define __MatrizDispersa__


#include "MatrizBandDisp.hpp"


/// Clase para el trabajar con matrices dispersas de punto flotante basada en el algoritmo Jagged Diagonal Storage (JDS)
/// El algoritmo esta optimizado para hacer producto matriz vector
/** @author Antonio Carrillo Ledesma
    @date primavera 2009
    @version 1.0.1
    @bug No hay errores conocidos
    @todo Hacer comportamiento para cambiar tamao de banda
    @todo Multiplicacin de matrices
*/
class MatrizDispersa : public MatrizBandDisp
{
private:

   /// Solicita la memoria necesaria para contener los valores de la matriz
   /** @param ren Nmero de renglones de la matriz
       @param col Nmero de columnas de la matriz
       @param ban Tamao de la banda  */
   void solicitaMemoria(const int ren, const int col, const int ban);


   /// Puntero a la matriz de datos
   ldouble **M;
   /// Arreglo que contiene los columnas de la matriz
   int **J;


public:

   /// Constructor de la clase
   /** @param ren Nmero de renglones de la matriz
       @param col Nmero de columnas de la matriz
       @param ban Tamao de la banda   */
   MatrizDispersa(const int ren, const int col, const int ban) : MatrizBandDisp()
   {
      solicitaMemoria(ren, col, ban);
      asignaNombre("SinNombre");
   }

   /// Constructor de la clase
   /** @param ren Nmero de renglones de la matriz
       @param col Nmero de columnas de la matriz
       @param ban Tamao de la banda
       @param nmb Nombre de la matriz*/
   MatrizDispersa(const int ren, const int col, const int ban, const char *nmb) : MatrizBandDisp()
   {
      solicitaMemoria(ren, col, ban);
      asignaNombre(nmb);
   }

   ~MatrizDispersa()
   {
      liberaMemoria();
   }

   /// Libera la memoria solicitada para la matriz
   void liberaMemoria(void);

   /// Inicializa la matriz al valor indicado
   /** @param val Valor por omisin para inicializar la matriz */
   void inicializa(ldouble val)
   {
      printf("\nFuncion no implementada en esta clase\n");
   }


   /// Asigna el valor indicado en el renglo y columna solicitado
   /** @param ren Renglon
       @param col Columna
       @param val Valor */
   void asigna(const int ren, const int col, const ldouble val);

   /// Retorna el nmero de columna cuando se para en el renglon e indice de la banda
   /** @param ren Nmero de renglon
       @param col Nmero de columna
       @return Nmero de columna cuando se para en el renglon e indice de la banda  */
   ldouble retorna(const int ren, const int col);

   /// Retorna el nmero de columnas de la banda para el renglon indicado
   /** @param ren Nmero de renglon
       @return Nmero de columnas de la banda para el renglon solicitado  */
   int retornaNumeroColumnasBanda(int ren);

   /// Retorna el nmero de columna cuando se para en el renglon e indice de la banda
   /** @param ren Nmero de renglon
       @param ind Nmero de indice
       @return Nmero de columna cuando se para en el renglon e indice de la banda  */
   inline int retornaNumeroColumna(int ren, int ind)
   {
      return (J[ren][ind]);
   }

   /// Retorna el valor de la columna cuando se para en el renglon e indice de la banda
   /** @param ren Nmero de renglon
       @param ind Nmero de indice
       @return Valor de la columna cuando se para en el renglon e indice de la banda  */
   inline double retornaValorColumna(int ren, int ind)
   {
      return (M[ren][ind]);
   }

   /// Multiplica la matriz por el escalar pasado como parametro
   /** @param esc Escalar */
   void multiplica(ldouble esc);

   /// Multiplica la matriz A por la matriz B
   /** @param a Puntero a matriz densa
       @param b Puntero a matriz densa  */
   void multiplica(MatrizDispersa *a, MatrizDispersa *b);

   /// Multiplica la matriz por el vector B dejando el Resultado en R
   /** @param b Puntero a un Vector
       @param r Puntero a un Vector */
   void multiplica(Vector *b, Vector *r);

   /// Visualiza las matrices internas usadas para soportar a las matrices bandadas
   void visualizaMatricesInternas(void);


};


/**
Esta clase implementa los componentes para el trabajar con matrices dispersas de punto flotante
@example EjemploMatrizDispersa.cpp */

#endif
